//
//  AppDelegate+notification.m
//  pushtest
//
//  Created by Robert Easterday on 10/26/12.
//
//

#import "AppDelegate+notification.h"
#import "PushPlugin.h"
#import <objc/runtime.h>

static char launchNotificationKey;

@implementation AppDelegate (notification)

- (id) getCommandInstance:(NSString*)className
{
  return [self.viewController getCommandInstance:className];
}

// its dangerous to override a method from within a category.
// Instead we will use method swizzling. we set this up in the load call.
+ (void)load
{
  Method original, swizzled;
  
  original = class_getInstanceMethod(self, @selector(init));
  swizzled = class_getInstanceMethod(self, @selector(swizzled_init));
  method_exchangeImplementations(original, swizzled);
}

- (AppDelegate *)swizzled_init
{ 
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(createNotificationChecker:)
                                               name:UIApplicationDidFinishLaunchingNotification object:nil];
  
//  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(onAppDidBecomeActive:)
//                                               name:UIApplicationDidBecomeActiveNotification object:nil];
  
  // This actually calls the original init method over in AppDelegate. Equivilent to calling super
  // on an overrided method, this is not recursive, although it appears that way. neat huh?
  return [self swizzled_init];
}

// This code will be called immediately after application:didFinishLaunchingWithOptions:. We need
// to process notifications in cold-start situations
- (void)createNotificationChecker:(NSNotification *)notification
{
  if(notification)
  {
    NSDictionary *launchOptions = [notification userInfo];
    if (launchOptions)
      self.launchNotification = [launchOptions objectForKey: @"UIApplicationLaunchOptionsRemoteNotificationKey"];
  }
}

- (void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken {
  PushPlugin *pushHandler = [self getCommandInstance:@"PushPlugin"];
  [pushHandler didRegisterForRemoteNotificationsWithDeviceToken:deviceToken];
}

- (void)application:(UIApplication *)application didFailToRegisterForRemoteNotificationsWithError:(NSError *)error {
  PushPlugin *pushHandler = [self getCommandInstance:@"PushPlugin"];
  [pushHandler didFailToRegisterForRemoteNotificationsWithError:error];
}

// this method is invoked when:
// - a regular notification is tapped
// - an interactive notification is tapped, but not one of its buttons
-(void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo fetchCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler{
  NSLog(@"didReceiveNotification");
  if (application.applicationState == UIApplicationStateActive) {
    PushPlugin *pushHandler = [self getCommandInstance:@"PushPlugin"];
    pushHandler.notificationMessage = userInfo;
    pushHandler.isInline = YES;
    [pushHandler notificationReceived];
  } else {
    //save it for later
    self.launchNotification = userInfo;
  }
  completionHandler(UIBackgroundFetchResultNewData);
}


// this method is invoked when:
// - one of the buttons of an interactive notification is tapped
// see https://developer.apple.com/library/mac/documentation/NetworkingInternet/Conceptual/RemoteNotificationsPG/Chapters/IPhoneOSClientImp.html#//apple_ref/doc/uid/TP40008194-CH103-SW1
- (void)application:(UIApplication *) application handleActionWithIdentifier: (NSString *) identifier forRemoteNotification: (NSDictionary *) notification completionHandler: (void (^)()) completionHandler {

  // the notification already contains the category, but the client also needs the identifier (action button)
  NSMutableDictionary *mutableNotification = [notification mutableCopy];

  [mutableNotification setObject:identifier forKey:@"identifier"];
  if (application.applicationState == UIApplicationStateActive) {
    PushPlugin *pushHandler = [self getCommandInstance:@"PushPlugin"];
    pushHandler.notificationMessage = mutableNotification;
    pushHandler.isInline = YES;
    [pushHandler notificationReceived];
  } else {
    PushPlugin *pushHandler = [self getCommandInstance:@"PushPlugin"];
    pushHandler.notificationMessage = mutableNotification;
    [pushHandler performSelectorOnMainThread:@selector(notificationReceived) withObject:pushHandler waitUntilDone:NO];
  }
  completionHandler();
}

- (void)onAppDidBecomeActive:(NSNotification*)notification {
  
  NSLog(@"active");
  //zero badge
  UIApplication *application = [UIApplication sharedApplication];
  application.applicationIconBadgeNumber = 0;
  if (self.launchNotification) {
    PushPlugin *pushHandler = [self getCommandInstance:@"PushPlugin"];

    pushHandler.notificationMessage = self.launchNotification;
    self.launchNotification = nil;
    [pushHandler performSelectorOnMainThread:@selector(notificationReceived) withObject:pushHandler waitUntilDone:NO];
  }
}

// The accessors use an Associative Reference since you can't define a iVar in a category
// http://developer.apple.com/library/ios/#documentation/cocoa/conceptual/objectivec/Chapters/ocAssociativeReferences.html
- (NSMutableArray *)launchNotification
{
  return objc_getAssociatedObject(self, &launchNotificationKey);
}

- (void)setLaunchNotification:(NSDictionary *)aDictionary
{
  objc_setAssociatedObject(self, &launchNotificationKey, aDictionary, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (void)dealloc
{
  self.launchNotification	= nil; // clear the association and release the object
}

@end
